@@ -444,16 +444,13 @@ class MappingValidator(BaseValidator):
 
     def add(self, *subvalidators, **kwargs):
         """
-        Add the given keyword arguments as field validators
+        Add the given positional/keyword arguments as field validators
         """
         for validator in subvalidators:
-            if validator.source is None:
-                self._subvalidators += [(None, validator)]
-            else:
-                self.__setitem__(validator.source, make_validator(validator))
+            self._subvalidators += [(validator.source, make_validator(validator))]
 
         for name, validator in kwargs.items():
-            self.__setitem__(name, make_validator(validator))
+            self.__setitem__(name, validator)
         return self
 
     def __setitem__(self, name, validator):
@@ -513,9 +510,11 @@ class MappingValidator(BaseValidator):
                     source = validator.source
 
                 subcontext.currentfield = source
-                subcontext.workingdata[name] = validator.process(
-                    value.get(source, Undefined), subcontext
-                )
+                if name in subcontext.workingdata:
+                    v = subcontext.workingdata[name]
+                else:
+                    v = value.get(source, Undefined)
+                subcontext.workingdata[name] = validator.process(v, subcontext)
 
             except ValidationError, e:
                 subcontext.workingdata[name] = Undefined
