@@ -176,7 +176,7 @@ class Date(DateTime):
         default=Undefined,
         message=None,
         required=True,
-        format='%d/%m%/y',
+        format='%d/%m/%y',
     ):
         super(Date, self).__init__(source, default, message, required, format)
 
